
// cf. netwerk/bas/public/nsNetError.h
const ERROR_FTP_LOGIN  = 2152398869;
const ERROR_FTP_CWD    = 2152398870;
const ERROR_FTP_PASV   = 2152398871;
const ERROR_FTP_PWD    = 2152398872;
const ERROR_FTP_LIST   = 2152398873;

var gPublishSiteData;
var gDefaultSiteIndex = -1;
var gDefaultSiteName;

var gSiteListUpdated = false;
var gFirstFileEntry  = null;
var gCurrentFilter   = "all";

var gFTPpending      = false;

var gCurrentSiteURL  = "";
var gLastDirOpenOrClose = null;

function GetCurrentEditorFromSidebar()
{
  // Get the active editor from the <editor> tag
  // XXX This will probably change if we support > 1 editor in main Composer window
  //      (e.g. a plaintext editor for HTMLSource)

  // For dialogs: Search up parent chain to find top window with editor
  var editor;
  try {
    var editorElement = window.top.document.getElementById("content-frame");
    editor = editorElement.getEditor(editorElement.contentWindow);

    // Do QIs now so editor users won't have to figure out which interface to use
    // Using "instanceof" does the QI for us.
    editor instanceof Components.interfaces.nsIPlaintextEditor;
    editor instanceof Components.interfaces.nsIHTMLEditor;
  } catch (e) { dump (e)+"\n"; }

  return editor;
}

// Dialog initialization code
function Startup()
{

  if (!GetCurrentEditorFromSidebar())
  {
    dump("Publish: No editor or return data object not supplied\n");
    window.close();
    return;
  }

  gDialog.SiteTree       = document.getElementById("SiteTree");
  gDialog.SiteDropDown   = document.getElementById("SiteDropDown");
  gDialog.SiteList       = document.getElementById("SiteList");
  gDialog.DirectoryList  = document.getElementById("DirectoryList");
  gDialog.FilterDropDown = document.getElementById("FilterDropDown");
  gDialog.treeViewCheckbox = document.getElementById("treeViewCheckbox");
  gDialog.dirHierarchy   = document.getElementById("dirHierarchy");
  gDialog.dirHierarchyPopup = document.getElementById("dirHierarchyPopup");

  gPublishSiteData       = window.top.GetPublishSiteData();

  FillSiteList();

  SetWindowLocation();
}

function ShowDirHierarchy(url)
{
  _removeAllChildren(gDialog.dirHierarchyPopup);

  var URL = GetURLFromUrl(url);
  var path = URL.filePath;

  var dirArray = path.split("/");
  var menuitem = null, i, label = "";

  menuitem = document.createElementNS(XUL_NS, "menuitem");
  menuitem.setAttribute("label", "/");
  menuitem.setAttribute("value", "/");
  gDialog.dirHierarchyPopup.insertBefore(menuitem, gDialog.dirHierarchyPopup.firstChild);

  for (i=1; i< dirArray.length; i++)
  {
    label += "/" + dirArray[i];
    menuitem = document.createElementNS(XUL_NS, "menuitem");
    menuitem.setAttribute("label", label);
    menuitem.setAttribute("value", label);
    gDialog.dirHierarchyPopup.insertBefore(menuitem, gDialog.dirHierarchyPopup.firstChild);
  }
  if (menuitem)
    gDialog.dirHierarchy.selectedItem = menuitem;

  return;
}

function RemoveSubdirs(ti)
{
  // var selected = GetSelectedItem(gDialog.SiteTree);
  // ClearTreeSelection(gDialog.SiteTree);

  var tch = ti.lastChild;
  var ti = tch.lastChild;
  while (ti)
  {
    var tmp = ti.previousSibling;
    tch.removeChild(ti);
    ti = tmp;
  }

  // SelectTreeItem(gDialog.SiteTree, selected);
}

function ResetFirstFileEntry()
{
  gFirstFileEntry = null;
}

function onTreeModified(aEvent)
{
  if (aEvent.attrName == "open") {
    var e = aEvent.target;
    gLastDirOpenOrClose = e;
    var url = e.getAttribute("value");
    if (aEvent.newValue == "true")
    {
      if (!gDialog.treeViewCheckbox.getAttribute("checked"))
      {
        setTimeout("SwitchToOneDirView(\""+url+"\")", 500);
        return;
      }
      ResetFirstFileEntry();
      var foo = new FTPDirParser(url, e,
                                 AddFTPDirSubdirs, EndFtpRequest, ErrorFtpRequest);
      window.document.documentElement.setAttribute("style", "cursor: wait");
      AllowEvents(gDialog.SiteTree, false);
    }
    else
    {
      if (!gDialog.treeViewCheckbox.getAttribute("checked"))
      {
        setTimeout("SwitchToOneDirView(\""+url+"\")", 500);
        return;
      }
      RemoveSubdirs(e);
    }
  }
}

function SelectDirectory(e)
{
  var url = gCurrentSiteURL + e.selectedItem.value;
  setTimeout("SwitchToOneDirView(\""+url+"\")", 500);
}

function SwitchToOneDirView(url)
{
  ShowDirHierarchy(url);
  RemoveSubdirs(gDialog.SiteTree);
  e = gDialog.SiteTree;
  ResetFirstFileEntry();
  if (url != gCurrentSiteURL)
    AddFTPDirSubdirs(url, null, gDialog.SiteTree);
  var foo = new FTPDirParser(url, gDialog.SiteTree,
                             AddFTPDirSubdirs, EndFtpRequest, ErrorFtpRequest);
  window.document.documentElement.setAttribute("style", "cursor: wait");
  AllowEvents(gDialog.SiteTree, false);
}

function ErrorFtpRequest(url, status)
{
  var message = "";
  if (status == ERROR_FTP_LOGIN)
    message = _GetString("FtpLoginError");
  else if (status == ERROR_FTP_CWD)
    message = _GetString("FtpCwdError");
  else
    message = _GetString("FtpUnknownError");

  AlertWithTitle("FTP", message, window.top);

  EndFtpRequest();
}

function EndFtpRequest()
{
  window.document.documentElement.removeAttribute("style");
  EnableAllUI(true);
}

function AddFTPDirSubdirs(url, dirEntry, treeitem)
{
  var tch = treeitem.lastChild;

  var ti             = document.createElementNS(XUL_NS, "treeitem");
  var tr             = document.createElementNS(XUL_NS, "treerow");
  var tcLocation     = document.createElementNS(XUL_NS, "treecell");
  var tcSize         = document.createElementNS(XUL_NS, "treecell");
  var tcLastModified = document.createElementNS(XUL_NS, "treecell");

  var createdUpperDirEntry = false;

  if (dirEntry)
  {
    var location = dirEntry.location;
    var size;
    if (dirEntry.type == cnsIDirIndex.TYPE_DIRECTORY)
      size = "";
    else
      size = dirEntry.size;
    var description = dirEntry.description;
    ti.setAttribute("value", url + "/" + location);
    tcLocation.setAttribute("label", location);
  }
  else
  {
    var URL = GetURLFromUrl(url);
    var path = URL.filePath;
    if (path == "" || path == "/")
      return;

    // get rid of trailing slashes
    while (path.length && path[path.length - 1] == '/')
      path = path.substr(0, path.length - 1);

    // rely on nsIURL magic
    var dir = URL.directory;

    while (dir.length && dir[dir.length - 1] == '/')
      dir = dir.substr(0, dir.length - 1);

    URL.filePath = dir;

    size        = "";
    description = "";
    ti.setAttribute("value", URL.spec);
    tcLocation.setAttribute("label", "..");
    createdUpperDirEntry  = true;
  }

  tcSize.setAttribute("label", size);
  tcLastModified.setAttribute("label", description);
  
  tr.appendChild(tcLocation);
  tr.appendChild(tcSize);
  tr.appendChild(tcLastModified);
  ti.appendChild(tr);

  if (createdUpperDirEntry  ||
      (dirEntry &&
       dirEntry.type == cnsIDirIndex.TYPE_DIRECTORY))
  {
    ti.setAttribute("container", "true");
    var subtch = document.createElementNS(XUL_NS, "treechildren");
    ti.appendChild(subtch);

    tch.insertBefore(ti, gFirstFileEntry);
    gFirstFileEntry = ti.nextSibling;
    return;
  }

  tch.appendChild(ti);
  if (!gFirstFileEntry)
    gFirstFileEntry = ti;

  FilterEntry(gCurrentFilter, ti);
}



function FillSiteList()
{
  // Fill the site lists
  var count = 0;
  if (gPublishSiteData)
    count = gPublishSiteData.length;
  var i;

  if (!count)
  {
    var ti = document.createElementNS(XUL_NS, "menuitem");
    ti.setAttribute("label", "--");
    gDialog.SiteList.appendChild(ti);
    gDialog.SiteDropDown.setAttribute("disabled", true);
    gDialog.SiteDropDown.setAttribute("label", _GetString("NoSiteAvailable"));

    gDialog.FilterDropDown.setAttribute("disabled", true);
    return;
  }

  gDialog.SiteDropDown.removeAttribute("disabled");
  gDialog.SiteDropDown.removeAttribute("label");

  gDialog.FilterDropDown.removeAttribute("disabled");

  gDialog.SiteDropDown.removeAttribute("disabled");
  ti = document.createElementNS(XUL_NS, "menuitem");
  ti.setAttribute("label", "Choose a site");
  ti.setAttribute("disabled", "true");
  gDialog.SiteList.appendChild(ti);
  gDialog.SiteDropDown.selectedItem = ti;

  var ts = document.createElementNS(XUL_NS, "menuseparator");
  gDialog.SiteList.appendChild(ts);
  
  for (i = 0; i < count; i++)
  {
    var name = gPublishSiteData[i].siteName;

    ti = document.createElementNS(XUL_NS, "menuitem");
    ti.setAttribute("label", name);
    ti.setAttribute("value", _GetUrlForPasswordManager(gPublishSiteData[i]));
    ti.setAttribute("oncommand", "SelectSite(this)");

    gDialog.SiteList.appendChild(ti);
  }
}

function CheckIfSiteListUpdated()
{
  if (gSiteListUpdated)
  {
    _removeAllChildren(gDialog.SiteList);
    gPublishSiteData = GetPublishSiteData();
    FillSiteList();
    gSiteListUpdated = false;
  }
}

function TweakSiteSettings()
{
  gSiteListUpdated = true;
  window.top.openDialog("chrome://editor/content/EditorPublishSettings.xul","_blank", "chrome,close,titlebar,modal", "");
  CheckIfSiteListUpdated();
}

function _removeAllChildren(e)
{
  if (e)
  {
    var child = e.lastChild;
    while (child)
    {
      var tmp = child.previousSibling;
      e.removeChild(child);
      child = tmp;
    }
  }
}

function SelectSite(e)
{
  EnableAllUI(false);
  window.document.documentElement.setAttribute("style", "cursor: wait");
  gCurrentSiteURL = e.getAttribute("value");
  RemoveSubdirs(gDialog.SiteTree);

  ResetFirstFileEntry();
  var foo = new FTPDirParser(gCurrentSiteURL, gDialog.SiteTree, AddFTPDirSubdirs, EndFtpRequest, ErrorFtpRequest);

  gDialog.DirectoryList.addEventListener("DOMAttrModified", onTreeModified, false);
}

function _GetUrlForPasswordManager(publishData)
{
  if (!publishData || !publishData.publishUrl)
    return false;

  var url;

  // For FTP, we must embed the username into the url for a site address
  // XXX Maybe we should we do this for HTTP as well???
  if (publishData.username && GetScheme(publishData.publishUrl) == "ftp")
    url = _InsertUsernameIntoUrl(publishData.publishUrl, publishData.username,
                                 window.top.GetSavedPassword(publishData));
  else
    url = publishData.publishUrl;

  // Strip off terminal "/"
  var len = url.length;
  if (len && url.charAt(len-1) == "\/")
    url = url.slice(0, len-1);
  
  return url;
}

function _InsertUsernameIntoUrl(urlspec, username, passwd)
{
  if (!urlspec || !username)
    return urlspec;

  try {
    var ioService = GetIOService();
    var URI = ioService.newURI(urlspec, GetCurrentEditorFromSidebar().documentCharacterSet, null);
    URI.username = username;
    URI.password = passwd;
    return URI.spec;
  } catch (e) {}

  return urlspec;
}

function _GetString(name)
{
  if (!gStringBundle)
  {
    try {
      var strBundleService =
          Components.classes["@mozilla.org/intl/stringbundle;1"].getService(); 
      strBundleService = 
          strBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);

      gStringBundle = strBundleService.createBundle("chrome://editor/locale/sitemanager.properties"); 

    } catch (ex) {}
  }
  if (gStringBundle)
  {
    try {
      return gStringBundle.GetStringFromName(name);
    } catch (e) {}
  }
  return null;
}

function FilterEntry(filter, item)
{
  var isDir = (item.lastChild.nodeName == "treechildren");
  if (!isDir)
  {
    if (filter == "all")
      item.removeAttribute("hidden");
    else
    {
      var cell = item.firstChild.firstChild;
      if (filter == "html")
        var re = /\.html?$/i ;
      else if (filter == "images")
        re = /\.gif$|\.png$|\.jpg$|\.jpeg$/i ;
      if (re.test(cell.getAttribute("label")))
        item.removeAttribute("hidden");
      else
        item.setAttribute("hidden", "true");
    }
  }
}

function ChangeFilter(filter)
{
  var nodeList = document.getElementsByTagName("treeitem");
  var count = nodeList.length, i;
  for (i=0; i<count; i++)
  {
    var item  = nodeList.item(i);
    FilterEntry(filter, item);
  }
}

function SelectsFilter(e)
{
  gCurrentFilter = e.value;
  ChangeFilter(gCurrentFilter);
}


function openFile()
{
  var item = GetSelectedItem(gDialog.SiteTree);
  if (!item.getAttribute("container"))
  {
    alert(item.getAttribute("value"));
    window.top.editPage(item.getAttribute("value"), window.top, true);
  }
}

function GetSelectedItem(tree)
{
  if (tree.treeBoxObject.selection.count == 1)
    return tree.contentView.getItemAtIndex(tree.currentIndex);
  else
    return null;
}

// * selects a entry in the tree
//   param XULElement aItem
function SelectTreeItem(tree, aItem)
{
  /* first make sure item's containers are open */
  if (!aItem) return;
  var tmp = aItem.parentNode;
  while (tmp && tmp.nodeName != "tree") {
    if (tmp.nodeName == "treeitem")
      tmp.setAttribute("open", "true");
    tmp = tmp.parentNode;
  }

  /* then select the item */
  var itemIndex = tree.contentView.getIndexOfItem(aItem);
  tree.treeBoxObject.selection.select(itemIndex);
  /* and make sure it is visible in the clipping area of the tree */
  tree.treeBoxObject.ensureRowIsVisible(itemIndex);
}

function ClearTreeSelection(tree) {
  if (tree)
    tree.treeBoxObject.selection.clearSelection();
}

function EnableAllUI(enabled)
{
  if (enabled)
  {
    AllowEvents(gDialog.SiteTree, false);
    gDialog.SiteDropDown.removeAttribute("disabled");
    gDialog.DirectoryList.removeAttribute("disabled");
    gDialog.FilterDropDown.removeAttribute("disabled");
    gDialog.treeViewCheckbox.removeAttribute("disabled");
  }
  else
  {
    AllowEvents(gDialog.SiteTree, true);
    gDialog.SiteDropDown.setAttribute("disabled", "true");
    gDialog.DirectoryList.setAttribute("disabled", "true");
    gDialog.FilterDropDown.setAttribute("disabled", "true");
    gDialog.treeViewCheckbox.setAttribute("disabled", "true");
  }
}

function AllowEvents(tree, enabled)
{
  if (enabled)
    tree.removeAttribute("allowevents");
  else
    tree.setAttribute("allowevents", false);
}

function ToggleTreeView(checkbox)
{
  var treeView = checkbox.getAttribute("checked");
  if (treeView)
  {
    gDialog.dirHierarchy.setAttribute("value", "");
    var e = gDialog.SiteDropDown.selectedItem;
    SelectSite(e);
    return;
  }

  var item = GetSelectedItem(gDialog.SiteTree);

  if (item)
  {
    if (!item.getAttribute("container"))
    {
      var parentItem = item.parentNode.parentNode;
      if (parentItem.nodeName == "tree")
       var url = gCurrentSiteURL;
      else
        url = parentItem.getAttribute("value");
    }
    else
      url = item.getAttribute("value");
  }
  else if (gLastDirOpenOrClose)
    url = gLastDirOpenOrClose.getAttribute("value");
  else
    url = gDialog.SiteDropDown.selectedItem.getAttribute("value");

  // User wants a view of one directory only.... Let's switch
  // url contains the URL of the "current" directory
  RemoveSubdirs(gDialog.SiteTree);
  
  ResetFirstFileEntry();
  ShowDirHierarchy(url);
  AddFTPDirSubdirs(url, null, gDialog.SiteTree);
  var foo = new FTPDirParser(url, gDialog.SiteTree,
                             AddFTPDirSubdirs, EndFtpRequest, ErrorFtpRequest);
  window.document.documentElement.setAttribute("style", "cursor: wait");
  AllowEvents(gDialog.SiteTree, false);
}

function GetURLFromUrl(url)
{
  try {
    var URL = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURL);
    URL.spec = url;
    return URL;
  } catch (e) {
    return null;
  }
}
